export class Cat {
  constructor(x, y, targetSize) {
    this.x = x;
    this.y = y;
    this.size = 171;
    this.targetSize = targetSize;
    this.sprite = null;
    this.loaded = false;
    this.velocity = 24;

    this.steps = [];

    this.currentStepIndex = 0;
    this.isMoving = false;
    this.stepTimer = 0;
    this.stepDuration = 40; // frames to complete one movement
    this.moveDirection = null;


    loadImage('assets/cat.webp', (img) => {
      this.sprite = new Sprite(this.x, this.y, this.size, this.size);
      this.sprite.spriteSheet = img;
      this.sprite.anis.offset.y = 3;
      this.sprite.anis.frameDelay = 8;
      this.sprite.friction = 0;
      this.sprite.addAnis({
        jump: { row: 0, frames: 4 },
        death: { row: 1, frames: 4 },
        hurt: { row: 2, frames: 2 },
        idle: { row: 3, frames: 4 },
        walk: { row: 4, frames: 6 }
      });
      this.sprite.changeAni('idle');

      const scaleFactor = this.targetSize / 171;
      this.sprite.scale = scaleFactor;
      this.loaded = true;
    });
  }
  
  update() {
    // if (!this.sprite) return;
    if (!this.sprite || this.steps.length === 0) return;
    console.log(`updating sprite...`);

    if (this.isMoving) {
      this._continueMovement();
    } else if (this.currentStepIndex < this.steps.length) {
      this._startMovement(this.steps[this.currentStepIndex]);
    }

    this.sprite.position.x = this.x;
    this.sprite.position.y = this.y;

    // console.log(`steps to run: ${this.steps}`);

    this._handleInput();
    
  };

  update() {
    if (!this.sprite || this.steps.length === 0) return;
  
    if (this.isMoving) {
      this._continueMovement();
    } else if (this.currentStepIndex < this.steps.length) {
      this._startMovement(this.steps[this.currentStepIndex]);
    }
  
    this.sprite.position.x = this.x;
    this.sprite.position.y = this.y;
  }
  

  run(steps) {
    this.steps = steps.map(e => e.direction);
    this.currentStepIndex = 0;
    this.isMoving = false;
  }

  _startMovement(direction) {
    this.moveDirection = direction;
    this.stepTimer = 0;
    this.isMoving = true;
  
    if (direction === 'right') {
      this.changeAni('w');
    } else if (direction === 'up' || direction === 'down') {
      this.changeAni('j');
    }
  }

  _continueMovement() {
    this.stepTimer++;
  
    const delta = this.velocity / this.stepDuration;
  
    if (this.moveDirection === 'right') this.x += delta;
    if (this.moveDirection === 'up') this.y -= delta;
    if (this.moveDirection === 'down') this.y += delta;
  
    if (this.stepTimer >= this.stepDuration) {
      this.isMoving = false;
      this.currentStepIndex++;
      this.changeAni('i'); // back to idle after each move
    }
  }
  

  keyPressed(key) {
    console.log(`receiving keyboard input: ${key}`);
  }

  draw() {
    if (!this.sprite) return;
    this.update();
    this.sprite.update();
  };

  changeAni(key) {
    if (!this.sprite) return;

    switch (key) {
      case 'w':
        this.sprite.changeAni("walk");
        break;
      case 'j':
        this.sprite.changeAni("jump");
        break;
      case 'i':
        this.sprite.changeAni("idle");
        break;
      case 'h':
        this.sprite.changeAni("hurt");
        break;
      case 'd':
        this.sprite.changeAni("death");
        break;
      default:
        break;
    };
  };

  setPosition(x, y) {
    if (this.sprite) {
      this.sprite.position.x = x;
      this.sprite.position.y = y;
    };
  };

  remove() {
    if (!this.sprite) return;
    this.sprite.remove();
    this.sprite = null;
  }

  moveUp(){
    console.log(`sprite should move up...`);
    this.changeAni('j')
  }

  // helper
  _getSpritePosition(){
    console.log(`sprite's actual position: ${this.sprite.x}, ${this.sprite.y}`);
    return (this.sprite.x, this.sprite.y);
  }

  _translate(step){
    switch (step) {
      case 'up':
        console.log('move: up');
        return 'jump'
        break;
      case 'right':
        console.log('move: down');
        return 'walk'
        break;
      case 'down':
        console.log('move: up');
        return 'jump'
        break;
      default:
        break;
    }
  }

  _handleInput(){
    console.log('assume i have something....');
  }
}
  